export interface BaseLayoutOptions {
}
/**
 * All the layout classes are extended from this base layout class.
 */
export declare class BaseLayout extends EventTarget {
    /** Name of the layout. */
    protected readonly _name: string;
    /** Extra configuration options of the layout. */
    protected _options: BaseLayoutOptions;
    version: number;
    state: string;
    /**
     * Constructor of BaseLayout
     * @param  {Object} options extra configuration options of the layout
     */
    constructor(options?: BaseLayoutOptions);
    /**
     * @fires BaseLayout#onLayoutStart
     * @protected
     */
    _onLayoutStart(): void;
    /**
     * @fires BaseLayout#onLayoutChange
     * @protected
     */
    _onLayoutChange(): void;
    /**
     * @fires BaseLayout#onLayoutDone
     * @protected
     */
    _onLayoutDone(): void;
    /**
     * @fires BaseLayout#onLayoutError
     * @protected
     */
    _onLayoutError(): void;
    /**
     * Check the equality of two layouts
     * @param  {Object} layout The layout to be compared.
     * @return {Bool}   True if the layout is the same as itself.
     */
    equals(layout: BaseLayout): boolean;
    /** virtual functions: will be implemented in the child class */
    initializeGraph(graph: any): void;
    updateGraph(graph: any): void;
    start(): void;
    update(): void;
    resume(): void;
    stop(): void;
    getNodePosition(node: Node): [number, number];
    getEdgePosition(edge: any): {
        type: string;
        sourcePosition: number[];
        targetPosition: number[];
        controlPoints: any[];
    };
    /**
     * Pin the node to a designated position, and the node won't move anymore
     * @param  {Object} node Node to be locked
     * @param  {Number} x    x coordinate
     * @param  {Number} y    y coordinate
     */
    lockNodePosition(node: any, x: any, y: any): void;
    /**
     * Unlock the node, the node will be able to move freely.
     * @param  {Object} node Node to be unlocked
     */
    unlockNodePosition(node: any): void;
    _updateState(state: any): void;
}
