// deck.gl-community
// SPDX-License-Identifier: MIT
// Copyright (c) vis.gl contributors
import { BaseLayout } from '../../core/base-layout';
import { EDGE_TYPE } from '../../core/constants';
const defaultOptions = {
    nodePositionAccessor: (node) => [node.getPropertyValue('x'), node.getPropertyValue('y')]
};
export class SimpleLayout extends BaseLayout {
    _name = 'SimpleLayout';
    _graph = null;
    _nodeMap = {};
    _nodePositionMap = {};
    constructor(options = {}) {
        super({ ...defaultOptions, ...options });
    }
    initializeGraph(graph) {
        this.updateGraph(graph);
    }
    _notifyLayoutComplete() {
        this._onLayoutStart();
        this._onLayoutChange();
        this._onLayoutDone();
    }
    start() {
        this._notifyLayoutComplete();
    }
    update() {
        this._notifyLayoutComplete();
    }
    resume() {
        this._notifyLayoutComplete();
    }
    updateGraph(graph) {
        this._graph = graph;
        this._nodeMap = graph.getNodes().reduce((res, node) => {
            res[node.getId()] = node;
            return res;
        }, {});
        this._nodePositionMap = graph.getNodes().reduce((res, node) => {
            res[node.getId()] = this._options.nodePositionAccessor(node);
            return res;
        }, {});
    }
    setNodePositionAccessor = (accessor) => {
        this._options.nodePositionAccessor = accessor;
    };
    getNodePosition = (node) => this._nodePositionMap[node.getId()];
    getEdgePosition = (edge) => {
        const sourcePos = this._nodePositionMap[edge.getSourceNodeId()];
        const targetPos = this._nodePositionMap[edge.getTargetNodeId()];
        return {
            type: EDGE_TYPE.LINE,
            sourcePosition: sourcePos,
            targetPosition: targetPos,
            controlPoints: []
        };
    };
    lockNodePosition = (node, x, y) => {
        this._nodePositionMap[node.getId()] = [x, y];
        this._onLayoutChange();
        this._onLayoutDone();
    };
}
